package org.trypticon.snipsnap.commands;

import java.util.Set;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;
import java.util.ArrayList;
import java.util.List;
import java.util.Collections;

import org.trypticon.snipsnap.label.JabberIDLabel;
import org.trypticon.xmpp.command.AbstractCommandHandler;

import org.snipsnap.container.Components;
import org.snipsnap.snip.Snip;
import org.snipsnap.snip.SnipSpace;
import org.snipsnap.snip.Blog;
import org.snipsnap.snip.label.Label;
import org.snipsnap.snip.label.Labels;
import org.snipsnap.snip.label.SnipLabel;
import org.snipsnap.user.User;
import org.snipsnap.user.UserManager;
import org.snipsnap.app.Application;
import org.snipsnap.app.ApplicationManager;
import org.snipsnap.config.Configuration;
import org.snipsnap.config.ConfigurationManager;
import org.jabberstudio.jso.x.commands.CommandQuery;
import org.jabberstudio.jso.x.commands.CommandNote;
import org.jabberstudio.jso.x.xdata.XDataForm;
import org.jabberstudio.jso.x.xdata.XDataField;
import org.jabberstudio.jso.Packet;
import org.jabberstudio.jso.JID;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.PacketError;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.util.PacketException;
import org.jabberstudio.jso.util.Utilities;

/**
 * Ah-hoc command to post to a weblog.
 */
public class PostToWeblogCommand extends AbstractCommandHandler
{
    /**
     * The node of the command.
     */
    private static final String NODE = "http://trypticon.org/protocol/commands#post-weblog";

    /**
     * The name of the command.
     */
    private static final String NAME = "Post to Weblog";

    /**
     * Default constructor.
     */
    public PostToWeblogCommand()
    {
        super(NODE, NAME);
    }

    /**
     * Handles the command.
     *
     * @param request  the request query.
     * @param response the response query.
     * @throws PacketException if an error occurs handling the command.
     */
    public void handleCommand(CommandQuery request, CommandQuery response) throws PacketException
    {
        StreamDataFactory factory = request.getDataFactory();
        JID validUser = JID.valueOf("trejkaz@jabber.zim.net.au");

        // Cheap authentication.
        // TODO: Use normal lookups like the old message-based blogging system.
        JID senderJID = ((Packet) request.getParent()).getFrom().toBareJID();
        if (!senderJID.equals(validUser))
        {
            throw new PacketException(PacketError.AUTH, PacketError.NOT_AUTHORIZED_CONDITION);
        }

        // If the action is anything other than a cancel, this is the initial query.
        if (!"cancel".equals(request.getAction()))
        {
            // See if there is a form posted.
            StreamElement payload = request.getPayload();
            if (payload instanceof XDataForm)
            {
                XDataForm form = (XDataForm) payload;

                String title = form.getFieldValue("title");

                // The content field is tricky because it's sent one line per field.
                List<String> contentList = form.listFieldValues("content");
                StringBuffer content = new StringBuffer();
                for (String line : contentList)
                {
                    content.append(line).append("\n");
                }

                // Post if there is something to post... otherwise, fall through to display the form again.
                // TODO: Present some kind of notice that information was lacking.
                if (Utilities.isValidString(title) && Utilities.isValidString(content.toString()))
                {
                    if (postWeblogEntry(senderJID, title, content.toString()).isEmpty())
                    {
                        // TODO: Error.
                    }
                    else
                    {
                        response.setStatus(CommandQuery.COMPLETED);
                        response.addNote(CommandNote.INFO, "Successfully posted to Weblog!");
                        return;
                    }
                }
            }

            // There was no form, or there was a problem with the form.
            // Present the form to the user.
            /*
            CommandActions actions = new CommandActions();
            actions.setCompleteAvailable(true);
            actions.setType("complete");
            response.setActions(actions);
            */

            // Build up the data form which the user will need to fill out.
            XDataForm form = (XDataForm) factory.createExtensionNode(XDataForm.NAME);
            form.setTitle("Post to Weblog");
            form.setInstructions("Please enter the title and content of your post.");

            // TODO: User selection?

            // TODO: Weblog selection drop-down.  "list-single"

            XDataField titleField = form.addField("title", XDataField.TEXT_SINGLE);
            titleField.setLabel("Title");
            titleField.setRequired(true);

            XDataField bodyField = form.addField("content", XDataField.TEXT_MULTI);
            bodyField.setLabel("Content");
            bodyField.setRequired(true);

            response.setStatus(CommandQuery.EXECUTING);
            response.setPayload(form);
        }
        else // "cancel"
        {
            // TODO: Handle cancellation.
            // TODO: Consider moving this to the command manager.
        }
    }

    /**
     * Posts an entry to the weblog owned by the sending JID.
     *
     * @param senderJID the JID which sent the weblog entry.
     * @param title the title of the weblog entry.
     * @param content the content of the weblog entry.
     * @return the set of weblog names which were posted to.  Empty if no posts were made.
     */
    private Set postWeblogEntry(JID senderJID, String title, String content)
    {
        ApplicationManager applicationManager = (ApplicationManager) Components.getComponent(ApplicationManager.class);
        SnipSpace snipSpace = (SnipSpace) Components.getComponent(SnipSpace.class);
        UserManager userManager = (UserManager) Components.getComponent(UserManager.class);

        // Set which application we're looking at.  If we don't do this, we see no snips, and can't post.
        // The OID is like an ID for each SnipSnap instance running in the server.
        String oid = applicationManager.getApplication("/");
        Configuration applicationConfig = ConfigurationManager.getInstance().getConfiguration(oid);
        Application.get().setConfiguration(applicationConfig);
        Application.get().storeObject(Application.OID, oid);

        // TODO: Devise a more efficient means of searching.  Lucene (implement Indexable)?

        // First pass, find all the weblogs that we need to post to.
        Iterator snipIterator = snipSpace.getAll().iterator();
        Map<String, User> weblogs = new TreeMap<String, User>();
        while (snipIterator.hasNext())
        {
            Snip snip = (Snip) snipIterator.next();
            Labels labels = snip.getLabels();
            if (labels.getLabel("JabberID", senderJID.toString()) instanceof JabberIDLabel)
            {
                User user = userManager.load(snip.getName());

                // If it's null, the snip wasn't a user snip, so we ignore it.
                if (user != null)
                {
                    // Switch to the user whose page had the tag on it.
                    Application.get().setUser(user);
                    Label weblogLabel = labels.getLabel("Weblog");
                    if (weblogLabel instanceof SnipLabel)
                    {
                        Blog weblog = snipSpace.getBlog(weblogLabel.getValue());
                        if (weblog != null)
                        {
                            weblogs.put(weblog.getName(), user);
                        }
                    }
                }
            }
        }

        // Second pass to post to each of the weblogs.
        for (Map.Entry<String, User> entry : weblogs.entrySet())
        {
            String weblogName = entry.getKey();
            User user = entry.getValue();

            try
            {
                // Login the user...
                Application.get().setUser(user);

                Blog weblog = snipSpace.getBlog(weblogName);
                weblog.post(content, title);
            }
            finally
            {
                // Remove the user from the context as a precaution.
                Application.get().setUser(null);
            }
        }

        return weblogs.keySet();
    }
}
