/*
 * Copyright 2004-2005 Trypticon
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.trypticon.snipsnap.label;

import java.util.Map;

import org.snipsnap.snip.label.BaseLabel;
import org.snipsnap.snip.SnipFormatter;

/**
 * Label identifying the Jabber ID of a user.
 */
public class JabberIDLabel extends BaseLabel
{
    /**
     * Default constructor.
     */
    public JabberIDLabel()
    {
        super();
    }

    /**
     * Constructor setting the name and value.
     *
     * @param name the name to set.
     * @param value the value to set.
     */
    public JabberIDLabel(String name, String value)
    {
        super(name, value);
    }

    /**
     * Gets the type of the label.
     *
     * @return the label type.
     */
    public String getType()
    {
        return "JabberIDLabel";
    }

    /**
     * Generates the markup for displaying the input form for the label.
     *
     * @return the markup.
     */
    public String getInputProxy()
    {
        StringBuffer buffer = new StringBuffer();
        buffer.append("<table border=\"0\" cellpadding=\"0\" cellspacing=\"2\">");
        buffer.append("<tr><td>Jabber ID: </td><td>");
        buffer.append("<input type=\"text\" value=\"");
        buffer.append(value);
        buffer.append("\" name=\"jid\"/>");
        buffer.append("</td></tr></table>");
        return buffer.toString();
    }

    /**
     * Generates the text for displaying the label.
     *
     * @return the markup.
     */
    public String getListProxy()
    {
        StringBuffer buffer = new StringBuffer();
        buffer.append("<td>");
        buffer.append(name);
        buffer.append("</td><td>");
        buffer.append(SnipFormatter.toXML(getSnip(), "{xmpp:" + value + "}"));
        buffer.append("</td>");
        return buffer.toString();
    }

    /**
     * Handles the input from the user, setting the instance fields as appropriate.
     *
     * @param map the input map, containing the user's input into the fields.
     */
    public void handleInput(Map map)
    {
        setName("Jabber ID");
        setValue((String) map.get("jid"));
    }
}
