/*
 * Copyright 2004-2005 Trypticon
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.trypticon.snipsnap.macro;

import java.io.IOException;
import java.io.Writer;

import org.radeox.macro.BaseMacro;
import org.radeox.macro.parameter.MacroParameter;
import org.radeox.api.engine.ImageRenderEngine;
import org.radeox.api.engine.RenderEngine;
import org.radeox.api.engine.context.RenderContext;
import org.radeox.util.Encoder;

/**
 * Macro which will display a link to jabber.org JEP data.
 */
public class JepMacro extends BaseMacro
{
    /**
     * Executes the macro.
     *
     * @param writer the writer to append to.
     * @param params the parameters passed to the macro.
     * @throws java.io.IOException if there was an error appending to the writer.
     */
    public void execute(Writer writer, MacroParameter params)
        throws IOException
    {
        if (params.getLength() < 1)
        {
            throw new IllegalArgumentException("Missing parameter 1: JEP number");
        }

        StringBuffer tmp = new StringBuffer(params.get(0));
        while (tmp.length() < 4)
        {
            tmp.insert(0, "0");
        }

        // Shouldn't need to encode a number, but who knows what a user might try to put in.
        String jepNumber = Encoder.escape(tmp.toString());

        // Draw the link, mimicing the behaviour of LinkMacro.
        // Maybe we should extend from it or something...
        RenderContext context = params.getContext();
        RenderEngine engine = context.getRenderEngine();
        writer.write("<span class=\"nobr\">");
        if (engine instanceof ImageRenderEngine)
        {
            writer.write(((ImageRenderEngine) engine).getExternalImageLink());
        }
        writer.write("<a href=\"http://www.jabber.org/jeps/jep-");
        writer.write(jepNumber);
        writer.write(".html\">");
        writer.write("JEP-");
        writer.write(jepNumber);
        writer.write("</a></span>");
    }

    /**
     * Gets the name of the macro, which becomes the macro's prefix.
     *
     * @return the name of the macro.
     */
    public String getName()
    {
        return "jep";
    }

    /**
     * Gets the description of the macro.
     *
     * @return the description of the macro.
     */
    public String getDescription()
    {
        return "Formats a Jabber Enhancement Proposal (JEP.)";
    }

    /**
     * Gets the description of the macro's parameters.
     *
     * @return the description of the macro's parameters, each parameters in an element of the array.
     */
    public String[] getParamDescription()
    {
        String[] description =
        {
            "1: The JEP number"
        };
        return description;
    }
}
